
/*
 * lib.c
 *	subroutines needed by reap utility
 */

#include "reap.h"

/*
 * freeblox(devno)
 *
 * THIS WILL REQUIRE MODIFICATION TO WORK UNDER NON-SYSTEM-V UNIX.
 * IT SHOULD BE EASY.  PLEASE EMAIL WORKING MODS TO THE AUTHOR:
 *			dt@yenta.alb.nm.us
 *
 *	assuming devno is the device number of a file system,
 *	this function returns the free space on that file system,
 *	in blocks (whatever that means for that filesystem) as an int.
 *
 */

freeblox(devno)
int	devno;
{
	static struct ustat	ust;

	if (ustat(devno, &ust))
		ouch ("%s: ustat() failed\n");

	return (ust.f_tfree);

} /* freeblox() */




/*
 * parse(cmd)
 *	execute a line from the script file
 *
 * valid command lines:
 *	a blank line is ignored
 *	# comment (ignored)
 *	days filespecs
 *
 * filespecs is a comma separated list of any combination of the following:
 * 1. a directory (newsgroup) to include, without recursion.
 *	ex:  alt.sources.
 * 2. a directory to include, recursively.
 *	ex:  alt.sources
 * 3. a directory to exclude, without recursion.
 *	ex:  !alt.sex.
 * 4. a directory to exclude, recursively.
 *	ex:  !alt.sex
 */

parse(cmd)
char	*cmd;
{
	register int	l;
	int		recurseflag;
	double		days;
	char		*t,
			*p,
			*q,
			*seps = ",\n ";


/* skip initial whitespace */
	p = cmd;
	while (isspace(*p))
		++p;

/* ignore blank lines or comments */
	if (*p == '\0' || *p == '#')
		return(1);

/* read the expire time from the line as a floating point number,
 * then figure out what the timestamp on a file that old would be */
	age = now - (time_t) (atof(p) * SECINDAY);

/* skip ahead to the filespec list */
	while (!isspace(*p))
		++p;
	while (isspace(*p))
		++p;

/* initialize exclusion list to null list */
	preclude();

/* for each filespec in list */
	for (t = strtok(p, seps); t; t = strtok(NULL, seps)) {

	/* forbid starting with a slash */
		if (*t == '/')
			*t = '.';

	/* change dots to slashes except in column 1 */
		while ( (q = strchr(t+1,'.')) != NULL)
			*q = '/';

	/* for final dot or slash, remove it, and shut off recursion */
		recurseflag = 1;
		if (t[l = (strlen(t) - 1)] == '/') {
			recurseflag = 0;
			t[l] = '\0';
		}

		if (*t == '!')
			exclude (t+1, recurseflag);
		else
			include (t, recurseflag);
	}

	reap();

	return (0);

} /* parse() */



/*
 * newnode,exclude, include, preclude
 *
 *	functions to maintain global linked lists:
 *		incl	include this path in list of dirs to reap
 *		excl	leave out this directory
			(each entry can be recursive or not)
 *
 *	the functions are:
 *		include (text,rflag)	add to incl list
 *		exclude (text,rflag)	add to excl list
 *		preclude()		clear both lists
 */
struct filspec *
newnode(text)
char	*text;
{
	struct filspec	*f;

	if ( (f = (struct filspec *) malloc (sizeof(struct filspec))) == NULL ||
	    (f->name = malloc (strlen(text)+1)) == NULL)
		ouch ("%s: out of memory\n");

	strcpy (f->name, text);
	return (f);
}
exclude(text, rflag)
char	*text;
int	rflag;
{
	struct filspec	*f;

	f = newnode(text);
	f->recurse = rflag;
	f->next = excl;
	excl = f;
}
include(text, rflag)
char	*text;
int	rflag;
{
	struct filspec	*f;

	f = newnode(text);
	f->recurse = rflag;
	f->next = incl;
	incl = f;
}
preclude()
{
	struct filspec	*p;

	while (incl != NULL) {
		p = incl;
		incl = incl->next;
		free (p->name);
		free (p);
	}
	while (excl != NULL) {
		p = excl;
		excl = excl->next;
		free (p->name);
		free (p);
	}
}


/*
 * ouch(fmt, arg)
 *	outputs an error message and exits with error status
 */

ouch (fmt,arg)
char	*fmt, *arg;
{
	fprintf (stderr, fmt, progname, arg);
	exit (-1);
}
