
/*
 * main.c
 *	main routine and globals for reap utility, version 1.1
 */


#include "reap.h"


/* linked lists for included and excluded file specs */
struct filspec
		*incl = NULL,
		*excl = NULL
;
char
		*scriptfile = SCRIPT,	/* path of function script	*/
		*progname,		/* argv[0] for errors		*/
		*newsdir = NEWSDIR	/* news root directory		*/
;
int
		verbose = 0,		/* verbosity (-v) flag		*/
		dryrun = 0		/* do-not-unlink (-n) flag	*/
;
time_t
		age,			/* unlink files older than this	*/
		now			/* current time			*/
;


/************************
 *	MAIN FUNCTION	*
 ************************/

main(argc, argv)
int	argc;
char	*argv[];
{
	int		device,		/* device # containing newsdir	*/
			c,
			lineno = 0,
			summary = 0,
			thenfree,
			errflag = 0;
	long		wantblox;	/* desired free space		*/
	char		*p;
	static char	line[MAXLINE];
	FILE		*script;	/* file ptr for function script	*/
	struct stat	st;


/* save argv[0] for error spewings */
	progname = argv[0];

/* parse args */
	while ( (c=getopt(argc,argv,"vsnf:")) != EOF)
		switch(c) {
		case 'n':
			++dryrun;
			break;
		case 'v':
			++verbose;
			break;
		case 'f':
			scriptfile = optarg;
			break;
		case 's':
			++summary;
			break;
		default:
			++errflag;
			break;
		}
	
	if (argc - optind != 1 ||
	    sscanf(argv[optind], "%ld", &wantblox) != 1 )
		++errflag;

	if (errflag) {
		fprintf (stderr,
		    "usage: %s [-v] [-s] [-n] [-f funclist] freeblocks\n",
		    progname);
		exit (-1);
	}


/* stat newsdir to find the number of the device it's on */
	if (stat(newsdir, &st))
		ouch ("%s: can't stat %s\n", newsdir);
	device = st.st_dev;

/* open function script file for reading */
	if ( (script = fopen (scriptfile, "r")) == NULL)
		ouch ("%s: can't read %s\n", scriptfile);

/* Record the current time, for deciding what gets the axe. */
	time (&now);

/* chdir to newsdir */
	chdir (newsdir);

/* record initial freespace if summary is desired */
	if (summary)
		thenfree = freeblox(device);

/* main loop ... process until goal reached or end of script */

	while (freeblox(device) < wantblox &&
	    (p = fgets (line, MAXLINE, script)) != NULL )
		parse (line), ++lineno;

	fclose (script);

	if (summary) {
		c = freeblox(device);
		printf ("Freespace was %d, now %d.  Cleared %d.\n",
		    thenfree, c, c - thenfree);
		printf ("Stopped after line %d in %s\n", lineno,
		    scriptfile);
	}

/* return 0 if freespace goal was met at exit, 1 if not */
	exit (freeblox(device) >= wantblox ? 0 : 1);

} /* main() */

