
/*
 * reap.c
 *	contains the reap() function.
 *
 * Once the global linked lists incl and excl have been stuffed,
 * reap() actually scans the filesystem for files that meet the specs
 * and unlinks them if they are older than the age global variable.
 */

#include "reap.h"


reap()
{
	struct filspec	*f;

	for (f = incl; f != NULL; f = f->next) {
		if (verbose)
			printf ("scanning %s ...\n", f->name);
		dodir (f->name, f->recurse);
	}

} /* reap() */



dodir(name, rflag)
char	*name;
int	rflag;
{
	/* the following can be overwritten safely during recursion */
	static struct filspec	*e;
	static struct stat	st;
	static struct dirent	*dp;
	static char		thisname[MAXFILENAME+1];
	/* the following must be preserved through recursion */
	char			*fullpath;
	int			eflag = 0;
	DIR			*dirp;


/* open directory for reading */
	if ( (dirp = opendir(name)) == NULL)
		ouch ("%s: can't read directory %s\n", name);


/* see if this directory is excluded.
 * If it's excluded recursively, quit here.
 * If it's excluded non-recursively, set a flag, so we won't consider
 *	deleting any files in it, but we'll still explore subdirectories.
 */
	for (e = excl; e != NULL; e = e->next)
		if (!strcmp (name, e->name))
			break;
	if (e != NULL) {
		if (e->recurse)
			return;
		else
			++eflag;
	}


/* loop for each directory entry */
	while ( (dp = readdir(dirp)) != NULL) {

	/* name might be exactly MAXFILENAME characters long, and thus
	 * might not be null-terminated.  Some insurance:
	 */
		strncpy (thisname, dp->d_name, MAXFILENAME);
		thisname[MAXFILENAME] = '\0';

	/* skip dot and dotdot */
		if (!strcmp(thisname, ".") || !strcmp(thisname, ".."))
			continue;

	/* build the full pathname of current object */
		if ( (fullpath =
		    malloc(strlen(name)+strlen(thisname)+2)) == NULL)
			ouch ("%s: out of memory\n");

		sprintf (fullpath, "%s/%s", name, thisname);

	/* try to stat the object */
		if (stat(fullpath,&st)) {
			fprintf (stderr, "%s: can't stat %s\n",
			    progname, fullpath);
			free (fullpath);
			continue;
		}

	/* maybe recurse if it's a directory */
		if ( st.st_mode & S_IFDIR ) {
			if (rflag)
				dodir (fullpath, 1);
			free (fullpath);
			continue;
		}

	/* it's a file ...  is this a non-recursively excluded directory?
	 * if so, there's nothing to do to this file
	 */
		if (eflag)
			continue;

	/* leave it alone if this directory is excluded, or
	 * if it's new enough.
	 */
		if (eflag || st.st_mtime > age) {
			free (fullpath);
			continue;
		}

	/* reap this file! */
		if (dryrun) {
			printf ("Would unlink %s\n", fullpath);
			free (fullpath);
			continue;
		}
		if (verbose)
			printf ("Unlinking %s\n", fullpath);

		if (unlink (fullpath) == -1)
			fprintf (stderr,
			    "%s: cannot unlink %s\n", progname, fullpath);

		free (fullpath);

	} /* while */

	closedir (dirp);

} /* dodir() */
